const chalk = require('chalk')
const fs = require('fs')

const friendlyModuleParseErrorLabel = 'Module parse failed:'

function filterModuleParseError(message) {
    return message.indexOf(friendlyModuleParseErrorLabel) === -1
}

function formatMessage(message, isError) {
    let lines = message.split('\n')

    // remove relative path if exists
    // ./ss will be cut into ss and then matched into the first path
    // if it exists, then we don't want to show the relative path
    // TODO: find out a way to clean this up?
    if (lines[0].includes(lines[1].slice(2))) {
        lines.splice(1, 1)
    }

    if (lines.length > 2 && lines[1] === '') {
        // Remove extra newline.
        lines.splice(1, 1)
    }

    // webpack adds a list of entry points to warning messages
    // @ src/...
    // we don't need this since we already have the path printed out
    lines = lines.filter(line => line.indexOf(' @ ') !== 0)

    // line #0 is filename
    // line #1 is the main error message
    if (!lines[0] || !lines[1]) {
        return lines.join('\n')
    }

    lines[0] = chalk.underline.bold(lines[0])

    // Reassemble the message.
    message = lines.join('\n')

    return message.trim()
}

function formatWebpackMessages(json) {
    const formattedErrors = json.errors.map(function(message) {
        return formatMessage(message, true)
    })
    const formattedWarnings = json.warnings.map(function(message) {
        return formatMessage(message, false)
    })
    const result = {
        errors: formattedErrors,
        warnings: formattedWarnings
    }
    result.errors = result.errors.filter(filterModuleParseError)

    return result
}

module.exports = formatWebpackMessages
