const { resolve } = require('path')
const paths = require('./paths')
const nodeExternals = require('webpack-node-externals')
const formatTsLoaderMessages = require('./formatTsLoaderMessages')
const tsconfigPathsPlugin = require('tsconfig-paths-webpack-plugin')
const { tslintShouldEmitErrors } = require('./tsLintHelper')

module.exports = {
    mode: 'production',
    entry: paths.appIndexJs,
    target: 'node',
    externals: [nodeExternals()],
    output: {
        path: paths.appBuild,
        filename: 'bundle.prod.js',
        libraryTarget: 'commonjs'
    },
    resolve: {
        extensions: ['.ts', '.tsx', '.js', '.json'],
        plugins: [
            new tsconfigPathsPlugin({
                configFile: paths.appTsConfig
            })
        ]
    },
    module: {
        rules: [
            {
                test: /\.tsx?$/,
                enforce: 'pre',
                use: [
                    {
                        loader: require.resolve('tslint-loader'),
                        options: {
                            emitErrors: tslintShouldEmitErrors(paths.appTsLint),
                            tsConfigFile: paths.appTsConfig,
                            configFile: paths.appTsLint,
                            formatter: 'lintTable',
                            formattersDirectory: __dirname + '/formatters/'
                        }
                    }
                ]
            },
            {
                test: /\.tsx?$/,
                include: paths.appSrc,
                loader: require.resolve('ts-loader'),
                options: {
                    colors: true,
                    errorFormatter: formatTsLoaderMessages
                }
            }
        ]
    },
    optimization: {
        nodeEnv: false
    },
    node: {
        __dirname: false,
        __filename: false
    },
    performance: {
        hints: false
    }
}
