process.on('unhandledRejection', err => {
    throw err
})

const fs = require('fs-extra')
const spawn = require('child_process').spawnSync
const exec = require('child_process').execSync
const path = require('path')
const os = require('os')
const chalk = require('chalk')

let argv = process.argv.slice(2)

const createApp = appName => {
    const appPath = path.join(process.cwd(), appName)
    console.log(chalk.cyan('Creating ' + appName + '...'))
    const appPackageJson = {
        name: appName,
        version: '0.1.0',
        main: 'src/index.ts',
        private: true,
        scripts: {
            test: 'typescript-node-scripts test',
            start: 'typescript-node-scripts start',
            build: 'typescript-node-scripts build',
        },
    }

    const packageDependencies = [
        'typescript',
        'tslint',
        'typescript-node-scripts',
    ]
    const packageDevDependencies = ['@types/node', '@types/jest']

    console.log(chalk.cyan('Setting up template...'))
    fs.copySync(path.join(__dirname, '../template'), appPath)

    console.log(chalk.cyan('Creating package.json...'))
    fs.writeFileSync(
        path.join(appPath, 'package.json'),
        JSON.stringify(appPackageJson, null, 2) + os.EOL
    )
    fs.copyFileSync(
        path.join(__dirname, '../template/gitignore'),
        path.join(appPath, '.gitignore')
    )
    fs.unlinkSync(path.join(appPath, 'gitignore'))
    console.log(chalk.cyan('Installing dependencies...'))
    spawn('yarnpkg', ['add', ...packageDependencies], {
        cwd: appPath,
        stdio: 'inherit',
    })
    console.log(chalk.cyan('Installing dev dependencies...'))
    spawn('yarnpkg', ['add', ...packageDevDependencies, '--dev'], {
        cwd: appPath,
        stdio: 'inherit',
    })
    console.log(chalk.cyan('Initializing git repository...'))
    try {
        exec('git init', {
            cwd: appPath,
            stdio: 'ignore',
        })
        exec('git add -A', {
            cwd: appPath,
            stdio: 'ignore',
        })
        exec('git commit -m "first commit by typescript-node-scripts"', {
            cwd: appPath,
            stdio: 'ignore',
        })
    } catch (e) {
        console.error(
            chalk.red(
                'Failed to initialize git repository. You might want to initialize it yourself.'
            )
        )
    }
    console.log()
    console.log(chalk.green('Successfully initialized application!'))
    console.log(chalk.green('To start using your application, run: \n'))
    console.log('  cd ' + appName)
    console.log('  yarn start')
    console.log()
}

createApp(argv[0])
